@@ -101,6 +101,11 @@
  *    - Fix usage message
  *  -> 2.27
  * 
+ *  * 2002-10-04: Kurt Garloff <garloff@suse.de>
+ *    - Bugfix from Olaf Hering (don't use arg twice in sscanf)
+ *    - Multipathing support
+ *    - Fix Medium Changer (and other device types with spaces) detection 
+ *  -> 2.28
  */
 
 #include <stdio.h>
@@ -121,8 +126,8 @@
 #include <ctype.h>
 
 
-static char rcsid[] ="$Id: scsidev.c,v 1.28.2.19 2002/08/07 09:09:19 garloff Exp $";
-static char *versid = "scsidev " VERSION " 2002/08/07";
+static char rcsid[] ="$Id: scsidev.c,v 1.28.2.21 2002/10/04 12:42:09 garloff Exp $";
+static char *versid = "scsidev " VERSION " 2002-10-04";
 static char *copyright = "Copyright: GNU GPL  (see file COPYING)\n" \
 " (w)  1994--1997 Eric Youngdale <eric@andante.org>\n"\
 "      2000--2002 Kurt Garloff   <garloff@suse.de>";
@@ -158,6 +163,7 @@ int no_san = 0;
 int no_procscsi = 0;
 int nm_cbtu = 0;
 int supp_rmvbl = 0;
+int supp_multi = 0;
 int override_link_perm = 1;
 char * no_serial = "No serial number";
 unsigned long long no_wwid = 0;
@@ -1412,8 +1418,9 @@ int procscsi_parse (sname *spnt)
 	spnt->manufacturer = strdup (vendor);
 	spnt->model = strdup (product);
 	spnt->rev = strdup (rev);
-	sscanf (fourlnbuf[2], "  Type: %20s ANSI SCSI revision: %d",
+	sscanf (fourlnbuf[2], "  Type: %20c ANSI SCSI revision: %d",
 		devtype, &ansi);
+	devtype [20] = 0; rmv_trail_ws (devtype);
 	spnt->inq_devtp = linux_to_devtp (devtype);
 	if (!fourlnbuf[3][0])
 		return 0;
@@ -1594,8 +1601,7 @@ void create_partitions (sname * spnt)
 		ptr = fgets (pline, 128, pf);
 		if (!ptr)
 			break;
-		blk = sscanf (pline, " %d %d %d %8s", &maj, &min, &blk, nm);
-		if (blk < 4)
+		if (sscanf (pline, " %d %d %d %8s", &maj, &min, &blk, nm) < 4)
 			continue;
 		
 		/* Strip part */
@@ -1862,6 +1868,7 @@ void usage()
     fprintf (stderr, " -c mxms: Continue scanning until mxms missing devs found\n");
     fprintf (stderr, " -r     : trust Removeable media (only safe after boot)\n");
     fprintf (stderr, " -e     : use dEvfs like naming  (cbtu chars)\n");
+    fprintf (stderr, " -M     : support Multipathing: First device is aliased\n");
     fprintf (stderr, " -v/-q  : Verbose/Quiet operation\n");
     fprintf (stderr, " -h     : print Help and exit.\n");
 }
@@ -1882,7 +1889,7 @@ int main (int argc, char * argv[])
 	fprintf(stderr, DEVSCSI " either does not exist, or is not a directory\n");
 	exit(0);
     }
-    while ((c = getopt(argc, argv, "pflLvqshnderm:c:")) != EOF) {
+    while ((c = getopt(argc, argv, "pflLvqshnderMm:c:")) != EOF) {
 	switch (c) {
 	  case 'p':
 	    no_procscsi = 1; break;
@@ -1904,6 +1911,8 @@ int main (int argc, char * argv[])
 	    san_del = 1; break;
 	  case 'r':
 	    supp_rmvbl = 1; break;
+	  case 'M':
+	    supp_multi = 1; break;
 	  case 'e':
 	    nm_cbtu = 1; break;
 	  case 'v':
@@ -2334,12 +2343,17 @@ void build_special ()
 	     * case we find a duplicate.
 	     */
 	    if( match != NULL ) {
-		fprintf (stderr, "Line %d not matched uniquely\n", line);
-		fprintf (stderr, " Prev. match: %s\n", match->name);
-		fprintf (stderr, " Curr. match: %s\n", spnt->name);
-		break;
-	    }
-	    else
+		if (!supp_multi) {
+		    fprintf (stderr, "Line %d not matched uniquely\n", line);
+		    fprintf (stderr, " Prev. match: %s\n", match->name);
+		    fprintf (stderr, " Curr. match: %s\n", spnt->name);
+		    break;
+		} else {
+		    if (!quiet) 
+			fprintf (stderr, "Line %d: %s <=> %s\n",
+				 line, match->name, spnt->name);
+		}
+	    } else
 		match = spnt;
 	}
 
